import matplotlib.pyplot as plt
import numpy as np
import string
import time
from random import shuffle
from demo16 import selection_sort, insertion_sort, merge_sort

def generate_random_list(length):
    lst = [i for i in range(length)]
    shuffle(lst)
    return lst

def generate_dataset(start, end):
    file = open("sort_data.txt", "w")
    file.write("length,selection,insertion,merge\n")

    for length in range(start, end, start):
        lst = generate_random_list(length)
        file.write(str(length));

        for algorithm in [selection_sort, insertion_sort, merge_sort]:
            lst2 = lst.copy()
            start = time.time()
            algorithm(lst2)
            running_time = time.time() - start
            
            file.write("," + str(running_time))
        
        file.write("\n")

    file.close()
        

def plot_bar_graph():
    x = range(0,4)
    messages = [100, 250, 5, 212]
    bar_width = .35
    x2 = [0+bar_width,1+bar_width,2+bar_width,3+bar_width]
    messages2 = [25, 200,8,250]

    plt.bar(x, messages,bar_width)
    plt.bar(x2, messages2,bar_width)
    plt.xticks(x, ('Alice', 'Bob', 'Charlie', 'Eve'))

    plt.show()


def plot_random():
    x = list(range(1, 10))
    y = x.copy()
    shuffle(y)
    y2 = [2, 5, 4, 9, 8, 6, 100, 13, 109]

    plt.plot(x,y,"m-", label="y")
    plt.plot(x,y2,"r--", label = "y2")

    plt.legend()

    plt.xlabel("x axis")
    plt.ylabel("y axis")
    plt.title("Sample plot")

    plt.show()


def sort_values(d):  # helper function for plot_word_frequency, sorts dictionary by values
    val_list = []
    for v in d.values() :
        val_list.append(v)
    val_list.sort(reverse=True)

    return(val_list)


def plot_word_frequency(counts):
    # construct frequency dictionaries from word count dictionaries
    freq = {}
    for (title, d) in counts.items():
        freq[title] = {}

        # compute total number of words in title
        num_words = 0
        for v in d.values():
            num_words += v

        # compute frequencies
        for (k, v) in d.items():
            freq[title][k] = d[k]/num_words*100

    # sort by frequency
    sorted_1 = sort_values(freq["Frankenstein"])
    sorted_2 = sort_values(freq["Alice in Wonderland"])
    sorted_3 = sort_values(freq["A Tale of Two Cities"])
    sorted_4 = sort_values(freq["Pride and Prejudice"])

    # plot frequencies
    min = 0
    max = 2500
    x = range(min, max)
    y = sorted_1[min:max]
    plt.plot(x, y, label="Frankenstein")
    y = sorted_2[min:max]
    plt.plot(x, y, label="Alice in Wonderland")
    y = sorted_3[min:max]
    plt.plot(x, y, label="A Tale of Two Cities")
    y = sorted_4[min:max]
    plt.plot(x, y, label="Pride and Prejudice")
    plt.legend()

    # render graph
    plt.show()


def plot_vocabulary(word_list, counts):

    # data to plot
    labels = []
    data = {}
    for (title, count) in counts.items():
        labels.append(title)
        data[title] = []
        for word in word_list:
            if word in count.keys():
                data[title].append(count[word])
            else:
                data[title].append(0)

    # create plot
    n_words = len(word_list)
    n_books = len(labels)
    x_locations = np.arange(n_words)  # like a range, but we can do arithmatic with it (see line 115)
    bar_width = 0.35
    gap_width = .5*bar_width
    group_width = n_books*bar_width + gap_width
    colors = ['b','g','y','r']
    #assert(len(colors) == len(labels))

    rects = []
    for i in range(0,len(labels)):
        # plt.bar(x location, values, bar width (optional), flags (optional))
        rect = plt.bar((x_locations*group_width)+(i*bar_width), data[labels[i]],
                       width=bar_width, color=colors[i], label=labels[i])
        rects.append(rect)

    # define axes
    locations = []
    for i in range(n_words):
        locations.append(i*group_width+(n_books/2-.5)*bar_width)
    plt.xticks(locations, word_list)
    #plt.xlabel('Words')
    plt.ylabel('Frequency')

    # add title and legend
    plt.title('Word Frequency in Classic (a.k.a. Public Domain) Literature')
    plt.legend()

    # render graph
    plt.show()


def pixel_frequency(filename):
    f = open(filename,"r")

    # read ppm header
    header = []
    for i in range(3):
        header.append(f.readline())

    # read ppm body
    pixel_counts = {}
    lines = f.readlines()
    for line in lines:
        words = line.split()

        i = 0
        while(i < len(words)):
            pixel = 255*255*int(words[i])+255*int(words[i+1])+int(words[i+2])
            if pixel in pixel_counts.keys():
                pixel_counts[pixel] += 1
            else:
                pixel_counts[pixel] = 1
            i += 3
    return pixel_counts

def plot_ppm(filename):
    x = list(range(255*255*255 + 255*255+255+1))
    counts = pixel_frequency("ppms/"+filename)

    y = []
    for i in range(255*255*255 + 255*255+255+1):
        if i in counts.keys():
            y.append(counts[i])
        else:
            y.append(0)

    plt.plot(x, y, 'b-')
    plt.xlabel("x axis")
    plt.ylabel("y axis")
    plt.title("Pixel Frequency in "+filename)
    plt.legend()
    plt.show()


def color_saturation_frequency(filename, color):
    colors = ["r","g","b"]
    c_index = colors.index(color)

    f = open(filename,"r")

    # read ppm header
    header = []
    for i in range(3):
        header.append(f.readline())

    # read ppm body
    saturation_counts = {}
    lines = f.readlines()
    for line in lines:
        words = line.split()

        i = 0
        while(i < len(words)):
            sat = int(words[i+c_index])
            if sat in saturation_counts.keys():
                saturation_counts[sat] += 1
            else:
                saturation_counts[sat] = 1
            i += 3
    return saturation_counts


def plot_ppm_color(filename, color): # color should be "r", "g", or "b"
    x = list(range(256))
    counts = color_saturation_frequency("ppms/"+filename, color)

    y = []
    for i in range(256):
        if i in counts.keys():
            y.append(counts[i])
        else:
            y.append(0)

    plt.plot(x, y, 'b-')
    plt.xlabel("x axis")
    plt.ylabel("y axis")
    plt.title("Single Color Frequency in "+filename+" ("+color+")")
    plt.show()


def remove_middle_punctuation(s): # helper function for word_count(filename)
    return s.replace('—', ' ').replace('-', ' ').replace('\'','')#.replace('“', ' ').replace('”', ' ').replace('‘', ' ')\


def word_count(filename): # helper file for processing books in main()
    punctuation = string.punctuation + '“' + '”' + '‘' + '’'

    counts = {}
    num_words = 0
    f = open(filename, "r")
    lines = f.readlines()
    for line in lines:
        line = remove_middle_punctuation(line)
        words = line.split()
        for word in words:
            num_words += 1
            word = word.strip(punctuation)
            word = word.lower()
            if word in counts.keys():
                counts[word] = counts[word] +1
            else:
                counts[word] = 1

    return counts



def main():
    # sorting example
    generate_dataset(100,10000)


    # simple examples (uncomment to run)
    # plot_random()          # example line graph/scatterplot
    # plot_bar_graph()       # example bar graph

    # construct word_count dictionaries from files
    # and store in nested dictionary counts
    #counts = {}
    #counts["Frankenstein"] = word_count("books/84-0.txt")
    #counts["Alice in Wonderland"] = word_count("books/11-0.txt")
    #counts["A Tale of Two Cities"] = word_count("books/98-0.txt")
    #counts["Pride and Prejudice"] = word_count("books/1342-0.txt")

    # book word count analysis example: line/scatter plot
    #plot_word_frequency(counts)                                            # example with line/scatter plot
    #plot_vocabulary(["frankenstein", "hello", "world", "sister"], counts)  # example with bar graph

    # exercise from lecture
    #plot_ppm("group.ppm")  # line graph

    # practice problem from worksheet
    plot_ppm_color("group.ppm","r")


if __name__ == "__main__":
    main()
